@@ -15,6 +15,8 @@ import re
 from threading import Thread
 
 
+
+
 def decode_line(encoded):
 
     """Decodes a polyline that was encoded using the Google Maps method.
@@ -140,10 +142,10 @@ class googleRoutesearchDownloader(Thread):
             if str.find("Continue straight") == 0:
                 ico_name = "./plugins/ico_turn_32_32.png"
                 ico = gdk.pixbuf_new_from_file(ico_name)
-            elif str.find("Turn left") == 0 or str.find("Take the 1st left") == 0 or str.find("Take the 2nd left") == 0:
+            elif str.find("Turn left") == 0 or str.find("Take the 1st left") == 0 or str.find("Take the 2nd left") == 0 or str.find("Take the 3rd left") == 0:
                 ico_name = "./plugins/ico_turn_left_32_32.png"
                 ico = gdk.pixbuf_new_from_file(ico_name)
-            elif str.find("Turn right") == 0 or str.find("Take the 1st right") == 0 or str.find("Take the 2nd right") == 0 :
+            elif str.find("Turn right") == 0 or str.find("Take the 1st right") == 0 or str.find("Take the 2nd right") == 0 or str.find("Take the 3rd right") == 0:
                 ico_name = "./plugins/ico_turn_right_32_32.png"
                 ico = gdk.pixbuf_new_from_file(ico_name)
             elif str.find("Slight right") == 0:
@@ -158,7 +160,7 @@ class googleRoutesearchDownloader(Thread):
             elif str.find("Keep right") == 0:
                 ico_name = "./plugins/ico_turn_keep_right_32_32.png"
                 ico = gdk.pixbuf_new_from_file( ico_name)
-            elif str.find("Make a U-turn") == 0:
+            elif str.find("Make a U-turn") == 0 or str.find("Sharp left") == 0 or str.find("Sharp right") == 0:
                 ico_name = "./plugins/ico_turn_u_32_32.png"
                 ico = gdk.pixbuf_new_from_file( ico_name)
             self.geonames.markers.append([
@@ -196,6 +198,7 @@ class googleRoutesearchDownloader(Thread):
                                )
 
 
+
             t.add_point( point1 )
             self.geonames.points.append( [float(ell[0]),float(ell[1])] )
             self.osmMapa.osm.track_add( t )
@@ -206,11 +209,14 @@ class googleRoutesearchDownloader(Thread):
 
             step_count+=1
 
-        try:
-            i = gdk.pixbuf_new_from_file("./plugins/ico_comment_m.png")
-            self.geonames.icons.append( self.osmMapa.osm.image_add( float(ell[0]),float(ell[1]), i ) )
-        except:
-            pass
+        if step_count > 0:
+            ico = gdk.pixbuf_new_from_file("./plugins/ico_comment_m.png")
+            self.geonames.markers.append([
+                 float(ell[0]),float(ell[1]),
+                 "Destination point",
+                 ico,
+                 "./plugins/ico_comment_m.png"
+                 ])
 
         print "step added ", step_count
 
@@ -235,7 +241,7 @@ class googleRoutesearchDownloader(Thread):
             lat, lon = self.osmMapa.get_may_current_ll()
             end = "loc:%f+%f"%( float(lat), float(lon)  )
 
-        req = "/maps/api/directions/xml?mode=%s&origin=%s&destination=%s&sensor=true" % (self.geonames.mode[self.CONFIG['google_routesearch_mode']],start,end)
+        req = "/maps/api/directions/xml?mode=%s&origin=%s&destination=%s&sensor=true&language=en" % (self.geonames.mode[self.CONFIG['google_routesearch_mode']],start,end)
         print req
         req = req.replace(" ", "%20")
         conn.request("GET", req)
@@ -290,6 +296,7 @@ class google_routesearch:
 
         self.icons = []
 
+
         self.my_point = -1
 
         self.zoom_minimum = 15
@@ -299,12 +306,36 @@ class google_routesearch:
         self.espeak_say = ""
         self.espeak_dis = 0
         self.espeak_time = 0
+        self.espeak_langs = []
 
         if os.path.isfile("/usr/bin/espeak"):
             self.espeak = 1
+
+            os.system( "espeak --voices | awk '{print $2}' | grep -v \"Language\" > /tmp/espeak_langs" )
+            f = open( "/tmp/espeak_langs", "r" )
+            l = 1
+            while l:
+                try:
+                    l = f.readline().replace("\n","")
+                    if l<>"":
+                        #print "espeak lang add[",l,"]"
+                        self.espeak_langs.append(l)
+                except:
+                    pass
+
+            if len(self.espeak_langs) == 0:
+                self.CONFIG['google_routesearch_speak_street_lang'] = "en"
+
             print "google_routesearch - found espeak yyyy cool"
+
+            try:
+                print "google_routesearch speak street lang:", self.CONFIG['google_routesearch_speak_street_lang']
+            except:
+                self.CONFIG['google_routesearch_speak_street_lang'] = "en"
         else:
             self.espeak = 0
+
+
         try:
             print "google_routesearch speak:", self.CONFIG['google_routesearch_speak']
         except:
@@ -341,9 +372,28 @@ class google_routesearch:
 
 
     def espeak_cmd(self, str):
+
         print "---- google_routesearch espeak ----"
         print "---- speak [%s] ----" % str
-        os.system( ("espeak \"%s\""%( str)) )
+
+        if str.find( " onto " ) <> -1:
+            tmp = []
+            tmp = str.split("onto")
+            cmd = "espeak \"%s onto\" && espeak -v %s \"%s\" &" %( tmp[0],self.CONFIG['google_routesearch_speak_street_lang'], tmp[1])
+        elif str.find( " toward " ) <> -1:
+            tmp = []
+            tmp = str.split("toward")
+            cmd = "espeak \"%s toward\" && espeak -v %s \"%s\" &" %( tmp[0],self.CONFIG['google_routesearch_speak_street_lang'], tmp[1])
+        elif str.find( " to stay on " ) <> -1:
+            tmp = []
+            tmp = str.split(" to stay on ")
+            cmd = "espeak \"%s to stay on\" && espeak -v %s \"%s\" &" %( tmp[0],self.CONFIG['google_routesearch_speak_street_lang'], tmp[1])
+        else:
+            cmd = "espeak \"%s\""%( str)
+
+        print "[[[%s]]]"%cmd
+        #return None
+        os.system( cmd )
 
     def makeInit(self):
         pass
@@ -351,6 +401,34 @@ class google_routesearch:
     def get_name(self):
         return "Google - routesearch"
 
+    def dist_from_vector(self, pa,pb,pc):
+        tr = 0.0
+
+        a = (pb[1]-pa[1])/(pb[0]-pa[0])
+        c = pa[1]-( pa[0]*( a  ) )
+        b = (-pa[0]-c)/pa[1]
+
+        d = pa[0]+pb[1]+c
+        if d < 0:
+            d = -d
+        tr = d/( math.sqrt( (a*a)+(b*b) ) )
+
+        return tr
+
+    def angle_from_vector(self, pa, pb, pc):
+        a0 = self.osmMapa.cal_angle(pc[0],pc[1], pb[0],pb[1]  )
+        a1 = self.osmMapa.cal_angle(pa[0],pa[1], pb[0],pb[1]  )
+        res = a0[0]-a1[0]
+        if res == 0.0:
+            return 10.0
+
+        if res < 0:
+            res = -res
+        res -= 3.14159265
+        if res < 0:
+            res = -res
+        return res
+
     def do_draw(self, gpsmap, gdkdrawable):
         if self.osmMapa.osm.props.zoom>self.zoom_minimum:
 
@@ -382,14 +460,61 @@ class google_routesearch:
 
 
         if self.CONFIG['google_routesearch_speak'] and len(self.points) > 0 and self.CONFIG['gpslock']:
+            print "calculate route and espeak"
             cr = gdkdrawable.cairo_create()
             # find close point
             id_next = 0
             latc = self.osmMapa.osm.props.latitude
             lonc = self.osmMapa.osm.props.longitude
             dis_min = -10
+            dis_total = 0
+            dis_done = 0
             id = 0
+            p_old = [0.0,0.0]
             #print "search for close point------------"
+
+            for i,p in enumerate(self.points):
+                if i == 0:
+                    p_old = p
+                else:
+                    angle = self.angle_from_vector( [p[0],p[1]], [latc,lonc],[p_old[0],p_old[1]] )
+                    dis = self.osmMapa.distance( p[0],p[1], p_old[0],p_old[1] )
+                    dis_total+=dis
+
+                    if dis_min == -10:
+                        dis_min = angle
+                        id_next = i
+
+                    elif dis_min>=angle:
+                        dis_min = angle
+                        id_next = i
+                        dis_done = 0
+                    else:
+                        dis_done+=dis
+
+
+                    p_old = p
+            dis_done = dis_total-dis_done
+
+
+
+            """
+            for i,p in enumerate(self.points):
+                if i == 0:
+                    p_old = p
+                else:
+                    dis = self.dist_from_vector( [p[0],p[1]], [latc,lonc],[p_old[0],p_old[1]] )
+                    if dis_min == -10:
+                        dis_min = dis
+                        id_next = i
+
+                    elif dis_min>=dis:
+                        dis_min = dis
+                        id_next = i
+
+                        print "dis_min:[", dis_min,"] id:[",i,"]"
+                    p_old = p
+
             for p in self.points:
                 dis = self.osmMapa.distance( p[0],p[1], latc, lonc )
                 if dis_min == -10:
@@ -422,9 +547,9 @@ class google_routesearch:
             #p = self.points[id_next]
             #x,y = self.osmMapa.osm.geographic_to_screen(p[0],p[1])
             #self.osmMapa.draw_cloud( cr, x,y, ("clost point:\ndist:%f\n[%f,%f]"%(dis_min,p[0],p[1])) )
+            """
 
-            id_next-=1
-
+            #id_next-=1
             # next key turn
             #print "search for next key turn, close point:[",id_next,"]"
 
@@ -455,6 +580,10 @@ class google_routesearch:
 
                         info = "in: %s\n%s" % ( self.osmMapa.distance_to_string(dis_to_key), m[2] )
                         self.osmMapa.draw_cloud( cr, 50,5, info, 20, False)
+
+                        dist = "track: %s / %s " % (self.osmMapa.distance_to_string(dis_done), self.osmMapa.distance_to_string(dis_total) )
+                        self.osmMapa.draw_cloud( cr, 150,-40, dist, 15, False)
+
                         if m[4]<> None:
                             ico = cairo.ImageSurface.create_from_png( m[4] )
                             cr.set_source_surface( ico, 50,65 )
@@ -465,7 +594,7 @@ class google_routesearch:
                         if self.espeak_say <> m[2]:
                             self.espeak_say = m[2]
                             self.espeak_dis = 0.203
-                            self.espeak_time = t
+                            self.espeak_time = t+5
 
                             speak = "in %s %s " % ( self.osmMapa.distance_to_string(dis_to_key).replace("m","meters").replace("km", "kilometers"),  m[2])
                             self.espeak_cmd( speak )
@@ -474,20 +603,20 @@ class google_routesearch:
 
                                 if dis_to_key < 0.200 and self.espeak_dis > 0.200:
                                     self.espeak_dis = dis_to_key
-                                    self.espeak_time = t+2
+                                    self.espeak_time = t+4
 
                                     speak = "in %s %s " % ( self.osmMapa.distance_to_string(dis_to_key).replace("m","meters").replace("km", "kilometers"),  m[2])
                                     self.espeak_cmd( speak )
 
                                 elif dis_to_key < 0.100 and self.espeak_dis > 0.100:
                                     self.espeak_dis = dis_to_key
-                                    self.espeak_time = t+2
+                                    self.espeak_time = t+4
 
                                     speak = "in %s %s " % ( self.osmMapa.distance_to_string(dis_to_key).replace("m","meters").replace("km", "kilometers"),  m[2])
                                     self.espeak_cmd( speak )
                                 elif dis_to_key < 0.025 and self.espeak_dis > 0:
                                     self.espeak_dis = dis_to_key
-                                    self.espeak_time = t+2
+                                    self.espeak_time = t+30
 
                                     speak = "%s " % ( m[2])
                                     self.espeak_cmd( speak )
@@ -599,6 +728,15 @@ class google_routesearch:
                 iter+=1
         print "selected mode: ", self.mode[id]
         self.CONFIG['google_routesearch_mode'] = id
+    def on_changed_lang (self,picker):
+        str = picker.get_value()
+        for i, lang_name in enumerate(self.espeak_langs):
+            if lang_name == str:
+                self.CONFIG['google_routesearch_speak_street_lang'] = str
+                break
+
+        print "selected lang: ", self.CONFIG['google_routesearch_speak_street_lang']
+
 
     def on_bt_export(self,w, dialog):
         dialog.destroy()
@@ -680,6 +818,21 @@ class google_routesearch:
             hbox.pack_start(self.bt_speak, True)
 
 
+            pb_lang = hildon.PickerButton(gtk.HILDON_SIZE_AUTO,
+                                        hildon.BUTTON_ARRANGEMENT_VERTICAL)
+            pb_lang.set_title("lang")
+            selector_lang = hildon.TouchSelectorEntry(text=True)
+            id = 0
+            for i,lang_name in enumerate(self.espeak_langs):
+                selector_lang.append_text(lang_name)
+                if lang_name == self.CONFIG['google_routesearch_speak_street_lang']:
+                    id = i
+            pb_lang.set_selector(selector_lang)
+            pb_lang.set_active( id  )
+            pb_lang.connect("value-changed",self.on_changed_lang)
+            hbox.pack_start( pb_lang, True )
+
+
         """
         #sftp://yoyo@yoyo.ghost.pl/home/yoyo/yosmapa_builder/yosmapa_builder
         self.bt_auto = hildon.CheckButton(gtk.HILDON_SIZE_AUTO)
