@@ -152,11 +152,11 @@ class gpx_tracks:
         self.sec_range = [ 2, 5, 10, 20, 30, 60, 120]
         self.zoom_min = 10
         self.zoom_max = 16
+        self.from_zoom = 10
         self.tracks_gpx = []
         self.points_gpx = []
 
         self.osm_tracks = []
-        self.osm_images = []
 
         self.track_info_distans = 0.0
 
@@ -165,13 +165,9 @@ class gpx_tracks:
     def clear_my_layer(self):
         for t in self.osm_tracks:
             self.osmMapa.osm.track_remove(t)
-        for t in self.osm_images:
-            self.osmMapa.osm.image_remove(t)
-
         self.osm_tracks = []
-        self.osm_images = []
-
-
+        
+        self.osmMapa.marker_layer_clean( self.get_name() )
 
     def killIt(self):
         self.clear_my_layer()
@@ -305,7 +301,16 @@ class gpx_tracks:
         # points
         for p in self.points_gpx:
             img = gdk.pixbuf_new_from_file( p[3] )
-            self.osm_images.append( self.osmMapa.osm.image_add( p[0],p[1], img) )
+            #self.osm_images.append( self.osmMapa.osm.image_add( p[0],p[1], img) )
+            self.osmMapa.marker_add([
+                p[0],
+                p[1],
+                self.get_name(),
+                p[2],
+                img,
+                None,
+                self.from_zoom
+                ])
         print "track_point_rebuild DONE"
 
     def on_changed_sec (self,picker):
@@ -440,27 +445,14 @@ class gpx_tracks:
                 self.osmMapa.draw_arrow( gdkdrawable, ll[0], ll[1], [0.0, 0.0, 1.0, 0.7], [1.0,0.0,0.0, 0.9] )
 
 
-
+            """
             for p in self.points_gpx:
                 lt = p[0]
                 ln = p[1]
                 if t_ln < ln and b_ln > ln and t_lt > lt and b_lt < lt:
                     x,y = self.osmMapa.osm.geographic_to_screen(lt,ln)
                     self.osmMapa.draw_cloud( cr, x,y, p[2])
-
-
-
-
-
-
-
-
-
-    def do_render(self, gpsmap):
-        pass
-
-    def do_busy(self):
-        return False
+            """
 
     def do_button_press(self, gpsmap, gdkeventbutton):
         pass
\ No newline at end of file
