@@ -18,8 +18,16 @@ class tangogps_friendDownloader(Thread):
         t = data_res.split( "|" )
         for f in t:
             i = f.split("#")
-            if len(i) == 7:
-                friend = [ i[1], float(i[2]),float(i[3]) ]
+            if len(i) == 7 and float(i[2]) < 180 and float(i[2]) >-180 and float(i[3])<180 and float(i[3]) >-180:
+                friend = [
+                        float(i[2]),
+                        float(i[3]),
+                        self.pa.get_name(),
+                        i[1],
+                        self.pa.ico_friend,
+                        None,
+                        self.pa.from_zoom
+                        ]
                 self.pa.friend.append( friend )
         print "tangogps friend found ",len(self.pa.friend),"frend"
 
@@ -48,33 +56,43 @@ class tangogps_friend:
         self.osmMapa = osmMapa
 
         self.ico = "tangogps_ico.png"
+        self.ui_menu_main = 1
+        self.ui_menu_left = 0
+        self.ui_search = 1
+        self.from_zoom = 4
+        self.ui_thread = 0
+
+        self.ico_friend = gdk.pixbuf_new_from_file( './plugins/tangogps-friend.png' )
+
+        self.from_zoom = 3
 
         print "__init__ tangogps friend"
 
     def do_draw_friend(self):
-        for e in self.friend:
-            i = gdk.pixbuf_new_from_file( './plugins/tangogps-friend.png' )
-            self.images.append( self.osmMapa.osm.image_add(e[1], e[2],i) )
+        for p in self.friend:
+            self.osmMapa.marker_add([
+                        p[0],
+                        p[1],
+                        p[2],
+                        p[3],
+                        p[4],
+                        p[5],
+                        p[6]
+                        ])
 
     def clean_layer(self):
         self.friend = []
-        for i in self.images:
-            self.osmMapa.osm.image_remove( i )
-        self.images = []
+        self.osmMapa.marker_layer_clean( self.get_name() )
 
     def killIt(self):
-        self.clean_layer()
+        self.osmMapa.marker_layer_clean( self.get_name() )
 
     def makeInit(self):
         self.friend = []
-        self.images = []
         t = tangogps_friendDownloader( self )
         t.start()
-
         return self
 
-
-
     def on_bt_update(self, w, dialog):
         self.clean_layer()
         self.makeInit()
@@ -85,29 +103,17 @@ class tangogps_friend:
         bt_update.connect("clicked", self.on_bt_update, dialog)
         return bt_update
 
+    def search_query(self, str):
+        tr = []
+        for p in self.friend:
+            item_str = ("%s" % (p[3]))
+            if item_str.find(str) <> -1:
+                tr.append(p)
+        return tr
+
     def get_name(self):
         return "tangogps friend"
 
-    def do_draw(self, gpsmap, gdkdrawable):
-        if self.osmMapa.osm.props.zoom>10:
-            t_lt, t_ln, b_lt, b_ln = self.osmMapa.osm.get_bbox()
-            t_lt, t_ln, b_lt, b_ln = math.degrees(t_lt), math.degrees(t_ln), math.degrees(b_lt), math.degrees(b_ln)
-            cr = gdkdrawable.cairo_create()
-
-            for p in self.friend:
-                lt = p[1]
-                ln = p[2]
-                #print "lt:",t_lt,t_ln,"\t c:",lt,ln,"\tb:",b_lt,b_ln
-                if t_ln < ln and b_ln > ln and t_lt > lt and b_lt < lt:
-                    x,y = self.osmMapa.osm.geographic_to_screen(lt,ln)
-                    self.osmMapa.draw_cloud( cr, x,y, p[0])
-
-
-    def do_render(self, gpsmap):
-        pass
-
-    def do_busy(self):
-        return False
 
     def do_button_press(self, gpsmap, gdkeventbutton):
         pass
\ No newline at end of file
