@@ -61,8 +61,18 @@ gtk.gdk.threads_init()
 
 
 name = "yosmapa"
-version = "0.18"
+version = "0.19"
 """
+0.19
+    config store except broken file
+        no more config versioning
+    new search engin :) one for all :)
+    plugins rebuild
+    plugin[google_routesearch] new algoryth to search current vector
+        add destination paint
+        more key point know
+        street names now in configurable language
+        cloud about total track distance and done :)
 0.18-6
     repo clean up
     add repo UMP-PL http://ump.waw.pl/
@@ -154,7 +164,7 @@ version = "0.18"
 
 
 def deb(str):
-    print "-------------------------\n",str,"\n-------------------------"
+    print " //-------------------------\n||",str,"\n \\\\-------------------------"
 
 
 #Try static lib first
@@ -223,15 +233,21 @@ maps_source = [
                ]
 
 
+
+
+
+
+
+
 try:
     from win32com.shell import shellcon, shell
-    home = ("%s\\.%s_%s.conf" % (hell.SHGetFolderPath(0, shellcon.CSIDL_APPDATA, 0, 0), name,version) )
+    home = ("%s\\.%s_%s.conf" % (hell.SHGetFolderPath(0, shellcon.CSIDL_APPDATA, 0, 0), name,"0.18") )
 except ImportError: # quick semi-nasty fallback for non-windows/win32com case
-    home =  ("%s/.%s_%s.conf"% (os.path.expanduser("~"), name,version) )
+    home =  ("%s/.%s_%s.conf"% (os.path.expanduser("~"), name,"0.18") )
 
 def config_save():
     global home
-    print "config - saveing"
+    deb( "config - saveing" )
     ts = []
     for b in CONFIG:
         ts.append([b, CONFIG[b]])
@@ -245,20 +261,275 @@ def config_load():
     global home
     global CONFIG
 
-    if os.path.exists(home):
-        print "config - loading ..."
-        f = open(home,"r")
-        tp = pickle.load(f)
-        f.close()
-        for b in tp:
-            CONFIG[b[0]] = b[1]
+    try:
+        if os.path.exists(home):
+            deb( "config - loading ..." )
+            f = open(home,"r")
+            tp = pickle.load(f)
+            f.close()
+            for b in tp:
+                CONFIG[b[0]] = b[1]
 
-        print "done"
-    else:
-        print "config - config file not found :/"
+            print "done"
+        else:
+            print "config - config file not found :/"
+            config_save()
+    except:
+        deb("config - found that file is broken :/ restoring")
+        os.system("dbus-send --print-reply --type=method_call --dest=org.freedesktop.Notifications /org/freedesktop/Notifications org.freedesktop.Notifications.SystemNoteDialog string:'Config file is broken :( Restoring settings to default.' uint32:0 string:'NAO OK!'")
         config_save()
+        config_load()
+
+    #print ("detect home path: [%s]" % home)
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+class SearchDialog( hildon.Dialog):
+    def __init__(self, osmMapa):
+        gtk.Dialog.__init__(self)
+        self.osmMapa = osmMapa
+
+        print "SearchDialog len:",len(self.osmMapa.show_find_results)
+        if len(self.osmMapa.show_find_results) > 0:
+            self.osmMapa.marker_clear_from_layer( self.osmMapa.show_find_results )
+            self.osmMapa.show_find_results = []
+            self.destroy()
+            return None
+        else:
+            self.build()
+
+    def build(self):
+        try:
+            print "SearchDialog search for:\n",CONFIG['search'],"\n-----------"
+        except:
+            CONFIG['search'] = ""
+
+        try:
+            print "SearchDialog recent search for:\n",CONFIG['recent_search'],"\n-----------"
+            i = len(CONFIG['recent_search'])
+        except:
+            CONFIG['recent_search'] = []
+
+        recent_max = 30
+        if len( CONFIG['recent_search'] ) > recent_max:
+            CONFIG['recent_search'].pop(  )
+
+        self.set_title("Search")
+        self.set_size_request(640, 600)
+
+
+        if len(CONFIG['recent_search'])>0:
+            mvbox = gtk.VBox()
+            panel = hildon.PannableArea()
+            panel.add_with_viewport(mvbox)
+            self.vbox.pack_start( panel, True,True)
+            panel.set_size_request(640, 280)
+
+            for s in CONFIG['recent_search']:
+                hbox = gtk.HBox()
+                #if s[0] <> "":
+                #    img = gtk.Image()
+                #    img.set_from_file(s[1])
+                #    hbox.pack_start(img,False)
+                b = gtk.Button(s[0])
+                b.connect("clicked", self.on_recent_click)
+                hbox.pack_start(b, True)
+                mvbox.pack_start(hbox, False)
+
+
+        hbox = gtk.HBox()
+
+        self.plugins_query = []
+
+        for p in self.osmMapa.find_plugins:
+            try:
+                if p[0].ui_search:
+                    if p[0].ico:
+                        try:
+                            if p[0].ui_thread:
+                                it_thread = 1
+                            else:
+                                it_thread = 0
+                        except:
+                            it_thread = 0
+                        if it_thread:
+                            self.plugins_query.append( [p[0], None ])
+                        else:
+                            self.plugins_query.append( [p[0] ])
+                        ico = gtk.Image()
+                        ico.set_from_file( "./plugins/%s"%p[0].ico )
+                        hbox.pack_start(ico,False)
+
+
+            except Exception, e:
+                print "SearchDialog plugins query init ERROR ",p[1],e
+
+
+
+        bt_c = gtk.Button("[x]")
+        hbox.pack_start(bt_c, False)
+
+        e_search = gtk.Entry()
+        hbox.pack_start(e_search,True)
+
+        bt_c.connect("clicked", self.on_clean, e_search)
+
+
+        bt_search = gtk.Button("Find")
+        bt_search.connect("clicked", self.on_search, e_search)
+        hbox.pack_start(bt_search,False)
+
+
+        self.vbox.pack_start(hbox,False)
+
+        self.show_all()
+        self.run()
+        self.destroy()
+
+
+    def add_item_to_box(self, i):
+        hbox = gtk.HBox()
+
+        l = gtk.Label("%s"% self.osmMapa.distance_to_string( self.osmMapa.distance( i[0],i[1] ) ) )
+        hbox.pack_start(l, False)
+
+        ico = gtk.Image()
+        ico.set_from_pixbuf( i[4] )
+        hbox.pack_start(ico,False)
+
+        l = gtk.Label( i[3] )
+        l.set_line_wrap(True)
+        hbox.pack_start(l,False)
+
+        l = gtk.Label( " " )
+        hbox.pack_start(l,True)
+
+        bt = gtk.Button("Action")
+        bt.connect("clicked", self.on_result_release, i)
+        hbox.pack_start(bt, False)
+
+        return hbox
+
+    def on_search(self, w, e_search = None):
+        if e_search:
+            CONFIG['search'] = e_search.get_text()
+        reTmp = []
+        reTmp.append( [CONFIG['search']] )
+        for r in CONFIG['recent_search']:
+            if r[0] <> CONFIG['search']:
+                reTmp.append( r )
+        CONFIG['recent_search'] = []
+        CONFIG['recent_search'] = reTmp
+
+        d = hildon.Dialog()
+        d.set_title("Results: [%s]"%CONFIG['search'])
+
+        mvbox = gtk.VBox()
+        panel = hildon.PannableArea()
+        panel.add_with_viewport(mvbox)
+        d.vbox.pack_start( panel, True,True)
+        panel.set_size_request(640, 600)
+
+        for p in self.plugins_query:
+
+            res = p[0].search_query( CONFIG['search'] )
+
+            hbox = gtk.HBox()
+
+            l = gtk.Label( "%s (%i):" % (p[0].get_name(), len(res)) )
+            hbox.pack_start(l,False)
+            l = gtk.Label()
+            hbox.pack_start(l,True)
+            mvbox.pack_start( hbox, False )
+
+            if len(p) == 2:
+                vbox = gtk.VBox()
+                p[1] = vbox
+                mvbox.pack_start( vbox, False)
+                self.make_delay_iter_for_ui_thread( p )
+
+            for i in res:
+                mvbox.pack_start(self.add_item_to_box(i),False)
+                self.osmMapa.show_find_results.append(  i )
+
+            h = gtk.HSeparator()
+            mvbox.pack_start(h,True)
+
+        #d.show_all()
+        self.destroy()
+        #d.run()
+        d.destroy()
+        self.osmMapa.recalculate_center_and_zoom()
+
+
+    def make_delay_iter_for_ui_thread(self, plugin):
+        print "make_delay_iter_for_ui_thread ",plugin[0].get_name()
+
+    def on_result_release(self, w, item):
+        print "item release:",item
+        m = gtk.Menu()
+        dis = self.osmMapa.distance_to_string( self.osmMapa.distance( item[0],item[1] ) )
+
+        mi = gtk.MenuItem("Distans it: %s___________" % dis)
+        m.append(mi)
+
+        mi = gtk.MenuItem("Center to___________")
+        #mi.connect("activate",self.on_set_center,c[4],c[5])
+        m.append(mi)
+
+        if CONFIG['POI']:
+            mi = gtk.MenuItem("Add as POI___________")
+            #mi.connect("activate",self.on_add_as_poi,c[4],c[5])
+            m.append(mi)
+
+        if CONFIG['Google - routesearch']:
+            mi = gtk.MenuItem("Route to it___________")
+            #mi.connect("activate",self.on_route_to_it,c[4],c[5])
+            m.append(mi)
+
+        m.show_all()
+        m.popup( None, None, None , 2, 0)
+
+
+    def on_recent_click(self,w):
+        CONFIG['search'] = w.get_label()
+        self.on_search(None)
+
+
+    def on_clean(self, w, e_search ):
+        e_search.set_text("")
+        CONFIG['search'] = ""
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
 
-    print ("detect home path: [%s]" % home)
 
 
 class MyDialog( hildon.Dialog ):
@@ -286,6 +557,7 @@ class MyDialog( hildon.Dialog ):
             else:
                 iter+=1
 
+
         if type( CONFIG['engine_name'] ) == int and type( maps_source[id][0] ) == int:
             print "selected source map: ", maps_source[id]," using fash switch"
             CONFIG['engine_name'] = maps_source[id][0]
@@ -347,6 +619,8 @@ Autor: Bartlomiej Ceglik <yoyoek[noSpam]wp.pl>""")
         pack = True
 
         mvbox = gtk.VBox()
+
+
         panel = hildon.PannableArea()
         panel.add_with_viewport(mvbox)
         self.vbox.pack_start(panel, pack)
@@ -366,7 +640,17 @@ Autor: Bartlomiej Ceglik <yoyoek[noSpam]wp.pl>""")
             except:
                 pass
 
-            if CONFIG[plugin_name] and w <> None and ico == None:
+            if (CONFIG[plugin_name] and w <> None and ico == None):
+                addIt = 1
+            else:
+                addIt = 0
+            try:
+                if plugin[0].ui_menu_main == 1 and CONFIG[plugin_name] and w <> None:
+                    addIt = 1
+            except:
+                pass
+
+            if addIt:
                 try:
                     pvbox = gtk.VBox()
                     label = gtk.Label( plugin[1]+":" )
@@ -528,6 +812,7 @@ Autor: Bartlomiej Ceglik <yoyoek[noSpam]wp.pl>""")
                     except Exception, e:
                         print "plugin state to active killIt ERROR",pn,e
                     break
+        self.osmMapa.set_plugins_searchable()
 
         dialog = MyDialog(self.osmMapa)
         self.destroy()
@@ -548,6 +833,11 @@ class YosmapaOSD(gobject.GObject, osmgpsmap.GpsMapLayer):
         self.pb_gps_on = cairo.ImageSurface.create_from_png( './imgs/i_gps_on.png' )
         self.pb_gps = cairo.ImageSurface.create_from_png( './imgs/i_gps.png' )
         self.pb_menu = cairo.ImageSurface.create_from_png( './imgs/ico_work_64_64.png' )
+
+        self.pb_find = cairo.ImageSurface.create_from_png( './imgs/ico_find_48_48.png' )
+        self.pb_clean = cairo.ImageSurface.create_from_png( './imgs/ico_clean_48_48.png' )
+        #self.show_find_results
+
         self.make_drawing = 1
 
     def do_draw(self, gpsmap, gdkdrawable):
@@ -557,6 +847,8 @@ class YosmapaOSD(gobject.GObject, osmgpsmap.GpsMapLayer):
                 t = time.time()
 
             self.size = gdkdrawable.get_size()
+            CONFIG['width'] = self.size[0]
+            CONFIG['height'] = self.size[1]
             x = self.size[0]-64
 
             cr = gdkdrawable.cairo_create()
@@ -566,9 +858,9 @@ class YosmapaOSD(gobject.GObject, osmgpsmap.GpsMapLayer):
 
             if gpsPresent:
                 if CONFIG['gpslock'] == 1:
-                    cr.set_source_surface( self.pb_gps_on, x,100 )
+                    cr.set_source_surface( self.pb_gps_on, x,80 )
                 else:
-                    cr.set_source_surface( self.pb_gps, x,100 )
+                    cr.set_source_surface( self.pb_gps, x,80 )
                 cr.paint()
 
                 try:
@@ -578,11 +870,18 @@ class YosmapaOSD(gobject.GObject, osmgpsmap.GpsMapLayer):
                     print "GPS manager: accurasy is insain (",self.accuracy,")"
 
 
-            cr.set_source_surface( self.pb_min, x,200 )
+            cr.set_source_surface( self.pb_min, x,160 )
             cr.paint()
             cr.set_source_surface( self.pb_menu, x,self.size[1]-100 )
             cr.paint()
 
+            if len(self.osmMapa.find_plugins) > 0:
+                if len(self.osmMapa.show_find_results) > 0:
+                    cr.set_source_surface( self.pb_clean, x+10,self.size[1]-160 )
+                else:
+                    cr.set_source_surface( self.pb_find, x+10,self.size[1]-160 )
+                cr.paint()
+
             plug_nr = 0
             for plugin in self.osmMapa.plugins_list:
                 plugin_name = plugin[1]
@@ -590,11 +889,15 @@ class YosmapaOSD(gobject.GObject, osmgpsmap.GpsMapLayer):
                     if len(plugin) == 3:
                         plugin.append( plugin[0].get_menu_widgets(self) )
                     if plugin[3] <> None:
-                        cr.set_source_surface( plugin[2], 2, (plug_nr*self.plugin_ico_h)+self.plugin_ico_yoffset )
-                        cr.paint()
-                        plug_nr+=1
-
-
+                        try:
+                            if plugin[0].ui_menu_left:
+                                cr.set_source_surface( plugin[2], 2, (plug_nr*self.plugin_ico_h)+self.plugin_ico_yoffset )
+                                cr.paint()
+                                plug_nr+=1
+                        except:
+                                cr.set_source_surface( plugin[2], 2, (plug_nr*self.plugin_ico_h)+self.plugin_ico_yoffset )
+                                cr.paint()
+                                plug_nr+=1
 
 
             if self.osmMapa.osm.props.tiles_queued != 0:
@@ -625,10 +928,10 @@ class YosmapaOSD(gobject.GObject, osmgpsmap.GpsMapLayer):
         x = self.size[0]-64
         #print "x start",x,"bt press at",gdkeventbutton.x
         if gdkeventbutton.x >= x:
-            if gdkeventbutton.y>10 and gdkeventbutton.y<74:
+            if gdkeventbutton.y>10 and gdkeventbutton.y<70:
                 print "bt plus"
                 self.osmMapa.zoom_in_clicked(None)
-            elif gdkeventbutton.y>100 and gdkeventbutton.y<164:
+            elif gdkeventbutton.y>80 and gdkeventbutton.y<140:
                 print "bt gps"
                 if CONFIG['gpslock']:
                     CONFIG['gpslock'] = 0
@@ -641,19 +944,32 @@ class YosmapaOSD(gobject.GObject, osmgpsmap.GpsMapLayer):
                             self.osmMapa.osm.set_center( gps.lat, gps.lon )
                     except:
                         print "gps get trac kupa"
-            elif gdkeventbutton.y>200 and gdkeventbutton.y<264:
+            elif gdkeventbutton.y>160 and gdkeventbutton.y<220:
                 print "bt min"
                 self.osmMapa.zoom_out_clicked(None)
+            elif len(self.osmMapa.find_plugins) >0 and gdkeventbutton.y>(self.size[1]-160) and  gdkeventbutton.y<(self.size[1]-105):
+                print "bt search"
+                dialog = SearchDialog(self.osmMapa)
+
             elif gdkeventbutton.y>(self.size[1]-100) and  gdkeventbutton.y<(self.size[1]-34):
                 print "bt menu"
                 dialog = MyDialog(self.osmMapa)
+                dialog.run()
+                dialog.destroy()
                 #print "response from dialog [",res,"]"
 
         elif gdkeventbutton.x <= 52:
             plug_nr = 0
             for plugin in self.osmMapa.plugins_list:
                 plugin_name = plugin[1]
-                if CONFIG[plugin_name]:
+                try:
+                    if plugin[0].ui_menu_left:
+                        probeIt = 1
+                    else:
+                        probeIt = 0
+                except:
+                    probeIt = 1
+                if CONFIG[plugin_name] and probeIt:
                     try:
                         ico = plugin[0].ico
                         w = plugin[0].get_menu_widgets(self)
@@ -712,6 +1028,10 @@ class YosmapaOSD(gobject.GObject, osmgpsmap.GpsMapLayer):
                         mi.connect("activate",self.on_route_to_it,c[4],c[5])
                         m.append(mi)
 
+                    #mi = gtk.MenuItem("___________Details")
+                    #mi.connect("activate",self.on_details,c[4], c[5], c[6])
+                    #m.append(mi)
+
                     m.show_all()
                     m.popup( None, None, None , 2, 0)
 
@@ -719,6 +1039,18 @@ class YosmapaOSD(gobject.GObject, osmgpsmap.GpsMapLayer):
             print "YosmapaOSD press in:",( time.time()-t )
         return False
 
+    def on_details(self, w, lat, lon, str):
+        point = []
+        for p in self.osmMapa.markers:
+            if p[0] == lat and p[1] == lon and p[3] == str:
+                point = p
+                break
+        for p in self.osmMapa.show_find_results:
+            if p[0] == lat and p[1] == lon and p[3] == str:
+                point = p
+                break
+        print "found details point",point
+
     def on_route_to_it(self, w, lat, lon):
         p = None
         for plugin in self.osmMapa.plugins_list:
@@ -813,16 +1145,18 @@ class UI(gtk.Window):
         self.connect('key-release-event', self.on_key_down)
 
         self.ben_iter = 100
+        self.show_find_results = []
+        self.find_plugins = []
 
         CONFIG['work_work_work'] = 1
 
         try:
-            print "track record chk: ",CONFIG['track_record']
+            print "__init__track record chk: ",CONFIG['track_record']
         except:
             CONFIG['track_record'] = 0
 
         try:
-            print "autorotate mode: ",CONFIG['autorotate']
+            print "__init__autorotate mode: ",CONFIG['autorotate']
         except:
             CONFIG['autorotate'] = 1
 
@@ -848,6 +1182,8 @@ class UI(gtk.Window):
 
         self.clikable_clouds = []
 
+        self.markers = []
+
 
         if autorotate:
             if CONFIG['autorotate']:
@@ -855,7 +1191,7 @@ class UI(gtk.Window):
             else:
                 initial_mode = FremantleRotation.NEVER
             self.window_rotate = FremantleRotation(name, self, version, initial_mode)
-            print "---------------------------------\nautorotate init!\n---------------------------------"
+            print "__init__ autorotate init!"
 
 
         try:
@@ -865,8 +1201,152 @@ class UI(gtk.Window):
 
         gobject.timeout_add(500, self.print_tiles)
 
+    def set_plugins_searchable(self):
+        self.find_plugins = []
+        for p in self.plugins_list:
+            try:
+                if p[0].ui_search and CONFIG[p[1]]:
+                    self.find_plugins.append(p)
+                    print "add searchable plugin,",p[1]
+            except:
+                pass
+
     def set_plugins_list(self, plugins_list):
         self.plugins_list = plugins_list
+        self.set_plugins_searchable()
+
+
+    """
+    -------------------------------------------------------------------
+    marker
+    p(
+0        lat (float),
+1        lon (float),
+2        plugin_source (string),
+3        title (string) || None,
+4        image (pixbuf),
+5        osm_point (gobject) || None,
+6        zoom (int) start draw from zoom
+    )
+    """
+
+    def recalculate_center_and_zoom(self):
+        if len(self.show_find_results)>0:
+            CONFIG['gpslock'] = 0
+            if len(self.show_find_results ) == 1:
+                p = self.show_find_results[0]
+                self.osm.set_center_and_zoom(p[0],p[1], 17 )
+            else:
+                t_lt, t_ln, b_lt, b_ln = 0.0, 0.0, 0.0, 0.0
+                for i,p in enumerate(self.show_find_results):
+                    p[6] = 0
+                    if i == 0:
+                        t_lt, t_ln, b_lt, b_ln = p[0],p[1],p[0],p[1]
+
+                    if t_lt > p[0]: t_lt = p[0]
+                    if b_lt < p[0]: b_lt = p[0]
+                    if t_ln > p[1]: t_ln = p[1]
+                    if b_ln < p[1]: b_ln = p[1]
+
+                print t_lt, t_ln, b_lt, b_ln
+
+                dis = self.distance( t_lt,t_ln, b_lt,b_ln )*1000
+                acc_scale = {
+                      18: 70,
+                      17: 100,
+                      16: 200,
+                      15: 500,
+                      14: 1000,
+                      13: 2000,
+                      12: 4000,
+                      11: 9000,
+                      10: 10000,
+                      9:  30000,
+                      8:  70000,
+                      7:  100000,
+                      6:  200000,
+                      5:  500000,
+                      4:  1000000,
+                      3:  2000000,
+                      2:  4000000
+                      }
+                zoom = 17
+                for key in acc_scale:
+                    print "dis[",dis,"] val[",acc_scale[key],"]"
+                    if acc_scale[key] <= dis:
+                        zoom = key
+                        break
+
+                latc, lonc = [(t_lt+b_lt)/2,(t_ln+b_ln)/2]
+                self.osm.set_center_and_zoom(latc,lonc,zoom )
+
+
+    def marker_draw(self,cr):
+
+        print "marker_draw"
+        self.clikable_clouds = []
+        t_lt, t_ln, b_lt, b_ln = self.osm.get_bbox()
+        t_lt, t_ln, b_lt, b_ln = math.degrees(t_lt), math.degrees(t_ln), math.degrees(b_lt), math.degrees(b_ln)
+        zoom =  self.osm.props.zoom
+
+        if len(self.show_find_results)>0:
+            self.marker_clear_from_layer( self.markers )
+            make_from = self.show_find_results
+        else:
+            make_from = self.markers
+
+        for m in make_from:
+            lt = m[0]
+            ln = m[1]
+            if m[6] < zoom and t_ln < ln and b_ln > ln and t_lt > lt and b_lt < lt:
+                if not m[5]:
+                    m[5] = self.osm.image_add( lt, ln, m[4] )
+                if zoom>10 and m[3]:
+                    self.draw_cloud_byll(cr, lt, ln, m[3])
+            else:
+                if m[5]:
+                    self.osm.image_remove(m[5])
+                    m[5] = None
+
+
+    def marker_clear_from_layer(self, from_what):
+        for m in from_what:
+            if m[5]:
+                self.osm.image_remove( m[5] )
+                m[5] = None
+
+    def marker_layer_clean(self, plugin = ""):
+        if plugin == "":
+            for m in self.markers:
+                if m[5]:
+                    self.osm.image_remove( m[5] )
+            self.markers = []
+        else:
+            mark_tmp = []
+            for m in self.markers:
+                if m[2] == plugin:
+                    if m[5]:
+                        self.osm.image_remove( m[5] )
+                else:
+                    mark_tmp.append(m)
+            self.markers = mark_tmp
+
+    def marker_add(self, marker):
+        self.markers.append( [marker[0], marker[1], marker[2], marker[3], marker[4], None, marker[6]] )
+
+    def marker_del(self, marker):
+        mark_tmp = []
+        for m in self.markers:
+            if m[0] == marker[0] and m[1] == marker[1] and m[2] == marker[2]:
+                if m[5]:
+                    self.osm.image_remove( m[5] )
+            else:
+                mark_tmp.append(m)
+        self.markers = mark_tmp
+
+   #-------------------------------------------------------------------
+
+
 
     def get_may_current_ll(self):
         if gpsPresent:
@@ -876,7 +1356,9 @@ class UI(gtk.Window):
 
         return [ self.osm.props.latitude, self.osm.props.longitude ]
 
-    def distance( self,lat1, lon1, lat2, lon2):
+    def distance( self,lat1, lon1, lat2 = None, lon2 = None):
+        if lat2 == None and lon2 == None:
+            lat2,lon2 = self.get_may_current_ll()
         try:
             theta = lon1 - lon2
             dist = math.sin(math.radians(lat1)) * math.sin(math.radians(lat2)) +  math.cos(math.radians(lat1)) * math.cos(math.radians(lat2)) * math.cos(math.radians(theta))
@@ -893,7 +1375,10 @@ class UI(gtk.Window):
             return "%i m"%dis
 
 
-    def cal_angle(self, lat0, lon0, lat1, lon1):
+    def cal_angle(self, lat0, lon0, lat1 = None, lon1 = None):
+        if lat1 == None and lon1 == None:
+            lat1,lon1 = self.get_may_current_ll()
+
         deltax = lat1-lat0
         deltay = lon0-lon1
         M_PI = 3.14159265
@@ -902,7 +1387,9 @@ class UI(gtk.Window):
         return [ angle_rad, 0 ]
 
 
-    def cal_dist(self, lat0, lon0, lat1, lon1):
+    def cal_dist(self, lat0, lon0, lat1 = None, lon1 = None):
+        if lat1 == None and lon1 == None:
+            lat1,lon1 = self.get_may_current_ll()
         return math.sqrt((lon0 - lon1)**2 + (lat0 - lat1)**2)
 
     def draw_arrow(self, gdkdrawable, lat, lon,rgba=[0.9,0.2,0.9,0.7], borderRgba=[0.0,0.4,0.4,0.3] ):
@@ -958,8 +1445,14 @@ class UI(gtk.Window):
         x,y = self.osm.geographic_to_screen(lat,lon)
         self.draw_cloud(cr, x, y, str, font_size, Fifek)
     def draw_cloud(self, cr, x, y, str, font_size = 13, Fifek = True):
-        if x<0 or y<0 or str == "":
+        if str == "":
             return None
+        if x<0:
+            print "inwert"
+            x = CONFIG['width']+x
+        if y<0:
+            print "inwert"
+            y = CONFIG['height']+y
 
         rlat, rlon = self.osm.get_co_ordinates(int(x), int(y))
         cr.select_font_face("Georgia")
@@ -1021,7 +1514,7 @@ class UI(gtk.Window):
             cr.show_text(str)
 
 
-        self.clikable_clouds.append( [ x,y,x+width,y+height, float(math.degrees(rlat)), float(math.degrees(rlon)) ] )
+        self.clikable_clouds.append( [ x,y,x+width,y+height, float(math.degrees(rlat)), float(math.degrees(rlon)), str ] )
 
         return True
 
@@ -1256,22 +1749,23 @@ class PluginsLayer(gobject.GObject, osmgpsmap.GpsMapLayer):
         self.make_draw = 1
 
     def do_draw(self, gpsmap, gdkdrawable):
-        if self.make_draw:
+        if self.make_draw and CONFIG['work_work_work']:
             if timing_plugins:
                 t = time.time()
 
-            if CONFIG['work_work_work']:
-                self.osmMapa.clikable_clouds = []
-                for plugin in self.plugins:
-                    if CONFIG[plugin[1]]:
-                        try:
-                            if timing_plugins:
-                                tp = time.time()
-                            plugin[0].do_draw(gpsmap, gdkdrawable)
-                            if timing_plugins:
-                                print "plugin ", plugin[1]," in ",(time.time()-tp)
-                        except Exception, e:
-                            print "PluginsLayer do_draw ERROR (",plugin[1],"):", e
+            self.osmMapa.marker_draw(gdkdrawable.cairo_create())
+
+
+            for plugin in self.plugins:
+                if CONFIG[plugin[1]]:
+                    try:
+                        if timing_plugins:
+                            tp = time.time()
+                        plugin[0].do_draw(gpsmap, gdkdrawable)
+                        if timing_plugins:
+                            print "plugin ", plugin[1]," in ",(time.time()-tp)
+                    except Exception, e:
+                        print "PluginsLayer do_draw ERROR (",plugin[1],"):", e
             if timing_plugins:
                 print "PluginsLayers in",(time.time()-t)
             self.make_draw = 0
